// include the library code:
#include <LiquidCrystal.h>
#include <StopWatch.h>

/*
This code for the portable alarm system,
a.k.a., Smart Shield, is open for public use.

The alarm goes off when motion detected. The LED/s,
and the LCD screen flash with ligh. The speaker makes 
a sound of alarm. The data of the motion detections is
sent via serial and serial1 communication.
5 timers are activated for the first 5 motion detections.
A UI enables the user to see the different timers 
on the LCD screen and to do basic functions, using the
keypad.

The project is using Arduino Leonardo card.

@author: Eran Perelman.
@date:      5 December 2012.

 */

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// initialize the diferrent timers
StopWatch sec(StopWatch::SECONDS);
StopWatch sec1(StopWatch::SECONDS);
StopWatch sec2(StopWatch::SECONDS);
StopWatch sec3(StopWatch::SECONDS);
StopWatch sec4(StopWatch::SECONDS);
StopWatch sec5(StopWatch::SECONDS);

int ledPin = 13;                // choose the pin for the LED

int inputPin = 2;               // choose the input pin (for PIR sensor) 

int pirState = LOW;             // no motion detected at start

int val = 0;                               // variable for reading the pin status

int pinSpeaker = 11;           //Set up a speaker on a PWM pin (digital 9, 10, or 11)

int PushButton = 12;   // choose the input pin (for a pushbutton)

int backLight = 10;         // the backLight of the LCD screen

int counter = 0;

int time;

char incomingByte;  // incoming data

boolean on = false;

void setup() {
  pinMode(ledPin, OUTPUT);      // declare LED as output 
    
  pinMode(pinSpeaker, OUTPUT); // declare speaker as output
  
  pinMode(inputPin, INPUT);     // declare sensor as input 
  
  pinMode(PushButton, INPUT);    // declare pushbutton as input
  
  pinMode(backLight, OUTPUT);  // declare backlight as output
  
  digitalWrite(backLight, HIGH);   // set up the LCD's number of columns and rows:
 
  lcd.begin(16, 2);                                     // print a message to the LCD.
  lcd.setCursor(0,0);
  lcd.print("Hello!");
   
   lcd.setCursor(0,1);                              // print silent mode status to the LCD
   if (digitalRead(PushButton) == LOW) lcd.print("Silent Mode: OFF");
   else lcd.print("Silent Mode: ON");
   delay(5000);
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Hello!");
          
    Serial1.begin(9600);                           // serial1 communication for bluetooth on Arduino Leonardo
    Serial.begin(9600);                           // serial communication
    sec.start();                                           // first timer is started on setup
    
         // This is intializing time before starting the alarm (delay for 15 seconds).  
    while (sec.elapsed() < 16) {
        lcd.setCursor(0,1);
        lcd.print("Active in: ");
        if (sec.elapsed() < 6) lcd.print((15 - sec.elapsed()));
        else {
          lcd.setCursor(11,1);
          lcd.print(" ");
          lcd.setCursor(12,1);
          lcd.print((15 - sec.elapsed()));
          lcd.setCursor(0,1);
        }
        delay(1000);
    }
    // Stabilizing message for the PIR Sensor for 15 seconds
    lcd.clear();
    lcd.setCursor(0,0);
   lcd.print("Hello!");
    lcd.setCursor(0,1);
    lcd.print("Stabilizing");    
    for (int j = 0; j < 3; j++){
      lcd.setCursor(11,1);
      lcd.print("     ");
      lcd.setCursor(11,1);
      lcd.print(".");
      delay(1000);
      lcd.print(".");
      delay(1000);
      lcd.print(".");
      delay(1000);
      lcd.print(".");
      delay(1000);
      lcd.print(".");
      delay(1000);
    }
    lcd.clear();
    
     sec.reset();      // reset for the first timer. 
     sec.start();      // start the first timer.    
}

void loop() {
    // This is the loop when the alarm is active (after 15 seconds).     
      on = true;
      
   // send data only when you receive data (ACK message):
  if (Serial1.available() > 0) {  // if the data came
    incomingByte = Serial1.read(); // read byte
    if(incomingByte == '2')
      Serial1.println("OK");  // print message
  }
                 
      val = digitalRead(inputPin);  // read input value 
      
     if (val == HIGH)  {                                                  // check if the input is HIGH 
    if (digitalRead(PushButton) == LOW) {         // check if Silent Mode isn't active
     
     digitalWrite(ledPin, HIGH);                           // turn LED or LEDs ON 
      playTone(300, 800);                                                 // play alarm tone on speaker
      
      digitalWrite(backLight, HIGH);
      
      lcd.setCursor(0,0);             // set message on LCD screen "INTRUDER ALERT!" 
      lcd.print("INTRUDER ALERT! ");
      lcd.setCursor(0,1);
      lcd.print("                ");

      delay(150);
      
      } else {
             digitalWrite(ledPin, LOW);                       // turn LED or LEDs OFF 
             playTone(0, 0);                                                   // turn speaker OFF 
      }
      
     if (pirState == LOW) {                                           // we have just turned on (interrupt)

      Serial1.println("Motion detected!");                // The output change, not the state, via bluetooth
      Serial.println("Motion detected!");                // The output change, not the state, via USB cable 
      
      if (counter == 0) sec1.start();              // This is the first interrupt timer
      if (counter == 1) sec2.start();              // This is the second interrupt timer
      if (counter == 2) sec3.start();              // This is the third interrupt timer
      if (counter == 3) sec4.start();              // This is the fourth interrupt timer
      if (counter == 4) sec5.start();              // This is the fifth interrupt timer

      counter++;                                                  // The interrupts' counter

        pirState = HIGH; 
    } 
    
    } else { 

    digitalWrite(ledPin, LOW);                      // turn LED or LEDs OFF 
    playTone(0, 0);                                                 // turn speaker OFF 

    delay(300);     

    if (pirState == HIGH){                                 // we have just turned off 

      Serial1.println("Motion ended!");              // The output change, not the state, via bluetooth 
      Serial.println("Motion ended!");                // The output change, not the state, via USB cable 
      
      lcd.clear();                                                         //clear the screen from "INTRUDER ALERT!"

      pirState = LOW;    
    } 
  } 
      
      // set message on LCD screen "Alarm is active!"  for 5 seconds.
     if (sec.elapsed() < 6){
        if (val == LOW){
          lcd.setCursor(0,0);                                     
          lcd.print("Alarm is active!");
          lcd.setCursor(0,1);
          lcd.print("                ");
        } else{
          digitalWrite(backLight, LOW);
          lcd.setCursor(0,0);             // set message on LCD screen "INTRUDER ALERT!" 
          lcd.print("INTRUDER ALERT! ");
          lcd.setCursor(0,1);
          lcd.print("                ");
        }
    }
      
      if (sec.elapsed() == 6) lcd.clear();         //clear the screen from "Alarm is active!"
      
      //delay for 5 seconds before turning backlight off
      if (sec.elapsed() >= 6) digitalWrite(backLight, LOW);
  
  // This is the UI (the input from the keypad).
  int  x = analogRead (0);
  
  KeypadPressed(x);
}

// UI (Keypad input) method.
void KeypadPressed(int x){
   
  // Pressing "Right" button.
  if (x < 100) {
         
    digitalWrite(backLight, HIGH);
        
    // Credits text animation.
    lcd.clear();
    lcd.setCursor(0,0);
    
    lcd.print ("C");
    delay(500);
    lcd.print ("r");
    delay(500);
    lcd.print ("e");
    delay(500);
    lcd.print ("d");
    delay(500);
    lcd.print ("i");
    delay(500);
    lcd.print ("t");
    delay(500);
    lcd.print ("s");
    delay(2000);
    digitalWrite(backLight, LOW);
    delay(500);
    digitalWrite(backLight, HIGH);
    delay(500);
    digitalWrite(backLight, LOW);
    delay(500);
    digitalWrite(backLight, HIGH);
    delay(500);
    digitalWrite(backLight, LOW);
    delay(500);
    digitalWrite(backLight, HIGH);
    delay(3000);
    
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print ("Smart Shield");
    delay(3000);
    
    for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    lcd.scrollDisplayLeft();
    delay(150);
  }
  
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print ("A portable");
    lcd.setCursor(0,1);
    lcd.print ("alarm system");
    delay(3000);
    
    for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    lcd.scrollDisplayRight();
    delay(150);
  }
    
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print ("Author:");
    lcd.setCursor(0,1);
    lcd.print ("Eran Perelman");
    delay(3000);
    
    for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    lcd.scrollDisplayLeft();
    delay(150);
  }
  
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print ("Date:");
    lcd.setCursor(0,1);
    lcd.print ("5 December 2012");
    delay(3000);
    
    for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    lcd.scrollDisplayRight();
    delay(150);
  }
  
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print ("Open for the");
    lcd.setCursor(0,1);
    lcd.print ("public domain");
    delay(3000);
    
    for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    lcd.scrollDisplayLeft();
    delay(150);
  }
  
    lcd.clear();
    
    sec.reset();      // reset for the first timer. 
    sec.start();      // start the first timer.    
  }
  // Pressing "Up" button.
  else if (x < 200) {
        
    lcd.clear();
    digitalWrite(backLight, HIGH);
    
    // Showing the different timers of the motion detections (last to first)
    if (counter == 0){
      lcd.setCursor(0,0);                                     
      lcd.print ("No motion");
      lcd.setCursor(0,1);
      lcd.print("detected!");
      delay(3000);
      lcd.clear();
      
    } else {
          
      int TempCounter;
      
      if (counter >= 5) TempCounter = 5;
      else TempCounter = counter;
      
      while (TempCounter > 0){
      lcd.setCursor(0,0);                                     
      lcd.print("Alert # ");
      lcd.print(TempCounter);
      
      if (TempCounter == 1) time = sec1.elapsed();
      if (TempCounter == 2) time = sec2.elapsed();
      if (TempCounter == 3) time = sec3.elapsed();
      if (TempCounter == 4) time = sec4.elapsed();
      if (TempCounter == 5) time = sec5.elapsed();
    
    lcd.setCursor(0, 1);
    lcd.print(time/3600);
    lcd.print("hr:");
    lcd.print((time /60) % 60);
    lcd.print("m:");
    lcd.print(time % 60);
    lcd.print("s");
    lcd.print(" ago");
    
    delay(3000);
    
    // scroll 16 positions (display length + string length) to the left
    // to move it back to center:
  for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    // scroll one position left:
    lcd.scrollDisplayLeft(); 
    // wait a bit:
    delay(150);
  }
    lcd.clear();
    TempCounter--;
    }
    }
    
    sec.reset();      // reset for the first timer. 
    sec.start();      // start the first timer.    
  }
// Pressing "Down" button
  else if (x < 400){
        
    lcd.clear();
    digitalWrite(backLight, HIGH);
    
    // Showing the different timers of the motion detections (first to last)
    if (counter == 0){
      lcd.setCursor(0,0);                                     
      lcd.print ("No motion");
      lcd.setCursor(0,1);
      lcd.print("detected!");
      delay(3000);
      lcd.clear();
      
    } else {
          
      int TempCounter;
      int p = 1;
      
      if (counter >= 5) TempCounter = 5;
      else TempCounter = counter;
      
      while (p < (TempCounter + 1)){
      lcd.setCursor(0,0);                                     
      lcd.print("Alert # ");
      lcd.print(p);
      
      if (p == 1) time = sec1.elapsed();
      if (p == 2) time = sec2.elapsed();
      if (p == 3) time = sec3.elapsed();
      if (p == 4) time = sec4.elapsed();
      if (p == 5) time = sec5.elapsed();
    
    lcd.setCursor(0, 1);
    lcd.print(time/3600);
    lcd.print("hr:");
    lcd.print((time /60) % 60);
    lcd.print("m:");
    lcd.print(time % 60);
    lcd.print("s");
    lcd.print(" ago");
    
    delay(3000);
    
    // scroll 16 positions (display length + string length) to the left
    // to move it back to center:
  for (int positionCounter = 0; positionCounter < 16; positionCounter++) {
    // scroll one position left:
    lcd.scrollDisplayLeft(); 
    // wait a bit:
    delay(150);
  }
    lcd.clear();
    p++;
    }
    }
             
    sec.reset();      // reset for the first timer. 
    sec.start();      // start the first timer.    
  }
  // Pressing "Left" button
  else if (x < 600){
        
    lcd.clear();
    digitalWrite(backLight, HIGH);
    
      // Showing number of detections
    lcd.setCursor(0,0);                                     
    lcd.print ("Number of motion");
    lcd.setCursor(0,1);                                     
    lcd.print ("detections: ");
    lcd.print (counter);

    delay(5000);
    sec.reset();      // reset for the first timer. 
    sec.start();      // start the first timer.    
  }
  // Pressing "Select" button
  else if (x < 800){
          
     // Pausing the alarm. 
     on = false;
      digitalWrite(backLight, HIGH);
      lcd.clear();
      lcd.print ("Alarm is off!");
      lcd.setCursor(0,1);
      lcd.print ("SELECT -> ON");
      delay(1000);
      
      while (on == false){
       // Waiting for input
        x = analogRead (0);
       if (x < 800) {
          on = true;
          lcd.clear();
          lcd.setCursor(0,0);
          lcd.print("Active in 15");
          lcd.setCursor(0,1);
          lcd.print("seconds");
          delay(15000);
          sec.reset();      // reset for the first timer. 
          sec.start();      // start the first timer.
      }  
      }
  }
  
}

// Speaker - sound alarm method.
void playTone(long duration, int freq) { 
// duration in mSecs, frequency in hertz 

    duration *= 1000; 

    int period = (1.0 / freq) * 1000000; 

    long elapsed_time = 0; 

    while (elapsed_time < duration) { 

        digitalWrite(pinSpeaker,HIGH); 

        delayMicroseconds(period / 2); 

        digitalWrite(pinSpeaker, LOW); 

        delayMicroseconds(period / 2); 

        elapsed_time += (period); 

    } 

}
